// Basic buildah operations for container management
use std::process::Command;
use crate::process::CommandResult;
use super::{BuildahError, Builder};


/// Execute a buildah command and return the result
///
/// # Arguments
///
/// * `args` - The command arguments
///
/// # Returns
///
/// * `Result<CommandResult, BuildahError>` - Command result or error
pub fn execute_buildah_command(args: &[&str]) -> Result<CommandResult, BuildahError> {
    // Get the current thread-local Builder instance if available
    let debug = thread_local_debug();
    
    if debug {
        println!("Executing buildah command: buildah {}", args.join(" "));
    }
    
    let output = Command::new("buildah")
        .args(args)
        .output();
    
    match output {
        Ok(output) => {
            let stdout = String::from_utf8_lossy(&output.stdout).to_string();
            let stderr = String::from_utf8_lossy(&output.stderr).to_string();
            
            let result = CommandResult {
                stdout,
                stderr,
                success: output.status.success(),
                code: output.status.code().unwrap_or(-1),
            };
            
            if debug {
                if !result.stdout.is_empty() {
                    println!("Command stdout: {}", result.stdout);
                }
                
                if !result.stderr.is_empty() {
                    println!("Command stderr: {}", result.stderr);
                }
            }
            
            if result.success {
                if debug {
                    println!("Command succeeded with code {}", result.code);
                }
                Ok(result)
            } else {
                let error_msg = format!("Command failed with code {}: {}",
                    result.code, result.stderr.trim());
                if debug {
                    println!("Command failed: {}", error_msg);
                }
                Err(BuildahError::CommandFailed(error_msg))
            }
        },
        Err(e) => {
            if debug {
                println!("Command execution failed: {}", e);
            }
            Err(BuildahError::CommandExecutionFailed(e))
        }
    }
}

// Thread-local storage for debug flag
thread_local! {
    static DEBUG: std::cell::RefCell<bool> = std::cell::RefCell::new(false);
}

/// Set the debug flag for the current thread
pub fn set_thread_local_debug(debug: bool) {
    DEBUG.with(|cell| {
        *cell.borrow_mut() = debug;
    });
}

/// Get the debug flag for the current thread
pub fn thread_local_debug() -> bool {
    DEBUG.with(|cell| {
        *cell.borrow()
    })
}

/// Execute a buildah command with debug output
///
/// # Arguments
///
/// * `args` - The command arguments
/// * `builder` - Reference to a Builder instance for debug output
///
/// # Returns
///
/// * `Result<CommandResult, BuildahError>` - Command result or error
pub fn execute_buildah_command_with_debug(args: &[&str], builder: &Builder) -> Result<CommandResult, BuildahError> {
    if builder.debug() {
        println!("Executing buildah command: buildah {}", args.join(" "));
    }
    
    let result = execute_buildah_command(args);
    
    if builder.debug() {
        match &result {
            Ok(cmd_result) => {
                if !cmd_result.stdout.is_empty() {
                    println!("Command stdout: {}", cmd_result.stdout);
                }
                
                if !cmd_result.stderr.is_empty() {
                    println!("Command stderr: {}", cmd_result.stderr);
                }
                
                println!("Command succeeded with code {}", cmd_result.code);
            },
            Err(e) => {
                println!("Command failed: {}", e);
            }
        }
    }
    
    result
}
